/*
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef HGL_COMMON_APPLICATION_H
#define HGL_COMMON_APPLICATION_H

#include <vector>

#include "iapplication.h"

namespace HGL {

namespace Common {

class ApplicationImpl;
class IApplicationObserver;

/**
 * @ingroup util
 * @brief Base class of a typical <em>HGL Suite</em> application.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT Application : public IApplication {
	DISALLOW_COPY_AND_ASSIGN(Application)
public:

	typedef std::vector<const IApplicationObserver *> OBSERVERS;

	/**
	 * @brief Constructor
	 *
	 * @note the GPGMe subsystem is only needed if signing key actions are neccessary
	 *
	 * @param obs an initial vector of @c IApplicationObserver to register
	 * @param signals if <code>true</code> than signal handling will be enabled
	 * @param initGPGMe if <code>true</code> than the GPGMe subsystem gets initialized
	 */
	Application(const OBSERVERS &obs, bool signals = true, bool initGPGMe = true);

	/**
	 * @overload
	 */
	Application(bool signals = true, bool initGPGMe = true);

	virtual ~Application();

	virtual int execute(int argc, const char **argv) = 0;

	void registerApplicationObserver(const IApplicationObserver *const) const;
	void unregisterApplicationObserver(const IApplicationObserver *const) const;

	bool hasTerminated() const;

	virtual bool notificationsAvailable() const;

	virtual void notifyInfo(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							const std::string &summary = std::string()) const;

	virtual void notifyWarn(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							const std::string &summary = std::string()) const;

	virtual void notifyError(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							 const std::string &summary = std::string()) const;

	virtual void notifyFatal(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							 const std::string &summary = std::string()) const;

private:
	ApplicationImpl *const _pimpl;
};

}

}

extern template class std::vector<const HGL::Common::IApplicationObserver *>;

#endif // HGL_COMMON_APPLICATION_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
