/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_BASEVECTOR2D_H
#define HGL_TYPE_BASEVECTOR2D_H

#include "floatutil.h"

namespace HGL {

namespace Type {

/**
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT BaseVector2D {
public:
	inline BaseVector2D(float x, float y) : m_x(x), m_y(y) {}
	virtual ~BaseVector2D() {}

	inline float getX() const {
		return m_x;
	}

	inline float getY() const {
		return m_y;
	}

	inline void setX(float x) {
		m_x = x;
	}

	inline void setY(float y) {
		m_y = y;
	}

	friend inline void swap(BaseVector2D &l, BaseVector2D &r) {
		using std::swap;

		swap(l.m_x, r.m_x);
		swap(l.m_y, r.m_y);
	}

	inline BaseVector2D &operator=(BaseVector2D o) {
		swap(*this, o);
		return *this;
	}

	inline bool operator<(const BaseVector2D &o) const {
		return Common::FloatUtil::equalFloat(m_x, o.m_x) ?
			   (Common::FloatUtil::equalFloat(m_y, o.m_y) ? false : m_y < o.m_y) : m_x < o.m_x;
	}

	inline bool operator==(const BaseVector2D &o) const {
		return Common::FloatUtil::equalFloat(m_x, o.m_x) &&
			   Common::FloatUtil::equalFloat(m_y, o.m_y);
	}

protected:
	float m_x;
	float m_y;
};

}

}

#endif // HGL_TYPE_BASEVECTOR2D_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
