/*
    Copyright (c) 2014, Heiko Schäfer <heiko@rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_COMMON_CRC_H
#define HGL_COMMON_CRC_H

#include <string>

#include <stdint.h>

#include "linkercontrol.h"

namespace HGL {

namespace Common {

class _EXPORT CRC {
	DISALLOW_COPY_AND_ASSIGN(CRC)
public:
	CRC();
	~CRC() _CONST;

	uint32_t checksum() const _PURE;
	CRC &operator<<(const std::string &data);

private:
	static void make_crc_table();
	uint32_t update_crc(const char *const buf, std::size_t len) const;

private:
	uint32_t m_crc;

	static uint32_t m_crc_table[256];
	static bool m_crc_table_computed;
};

}

}

#endif /* HGL_COMMON_CRC_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
