/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file defaultprerenderoutput.h
 * @brief Empty implementation of the HGL::ImageOutput::IPreRenderOutput interface
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.70
 **/

#ifndef HGL_IMAGEOUTPUT_DEFAULTPRERENDEROUTPUT_H
#define HGL_IMAGEOUTPUT_DEFAULTPRERENDEROUTPUT_H

#include "iprerenderoutput.h"

namespace HGL {

namespace ImageOutput {

/**
* @ingroup ioplugins
* @brief Empty implementation of the HGL::ImageOutput::IPreRenderOutput interface
*
* You can use this class if you are not interested in implementing all notifications
* in your plugin.
*
* The default iplemenentations are empty.
*
* See @ref iopluginsdetail for more information.
*
* @see REGISTER_IOPLUGIN
*
* @author Heiko Schäfer <heiko@hgl.rangun.de>
* @since 0.4.70
* @copyright BSD License
**/
class _EXPORT DefaultPreRenderOutput : public IPreRenderOutput {
	DefaultPreRenderOutput(const DefaultPreRenderOutput &);
	DefaultPreRenderOutput &operator=(const DefaultPreRenderOutput &);
public:
	DefaultPreRenderOutput();
	~DefaultPreRenderOutput();

	/**
	 * @name Default implementations of the information functions
	 * @{
	 */

	virtual std::string getOutputFilename() const;

	//@}

	/**
	 * @name Empty default implementations of bitmap render methods
	 * @{
	 */

	virtual bool writeImage(const ICanvas &, const std::string & = std::string()) const
	throw(ImageOutputException) _CONST;

	//@}

	/**
	 * @name Empty default implementations of pre-render notifications
	 * @{
	 */

	virtual void update(const RENDEREVENT) throw(ImageOutputException) _CONST;
	virtual void update(const ICanvas::TRANSFORMATIONINFO *const)
	throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const PointPrimitive *const) throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const LinePrimitive *const) throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const CurvePrimitive *const) throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const PolyPrimitive *const) throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const EllipsePrimitive *const)
	throw(ImageOutputException) _NONNULL(2) _CONST;
	virtual void update(const TextPrimitive *const) throw(ImageOutputException) _NONNULL(2) _CONST;

	//@}

protected:
	/**
	 * @brief Announces a (new) output filename
	 *
	 * A plugin may add a suffix or even alter the output filename at all.
	 * In this case the new filename can get announced to the interpreter,
	 * which will use it furthermore.
	 *
	 * @since 0.5.25
	 */
	void announceOutputFilename(const std::string &outputFilename) const;

private:
	mutable std::string m_outputFilename;
};

}

}

#endif // HGL_IMAGEOUTPUT_DEFAULTPRERENDEROUTPUT_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
