/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_DEFAULTPROJECTFACTORY_H
#define HGL_TYPE_DEFAULTPROJECTFACTORY_H

#include "projectfactorybase.h"
#include "ipoint.h"

namespace HGL {

namespace Type {

/**
 * @ingroup util
 * @brief Convenience class for implementing HGL::Type::IProjectFactory
 *
 * This class provides default implemenentations for the methods of HGL::Type::IProjectFactory.
 * They all simply throw HGL::Exception::UnsupportedTypeException. Override the methods you need.
 *
 * @see HGL::Type::IProjectFactory
 * @see HGL::Exception::UnsupportedTypeException
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT DefaultProjectFactory : public ProjectFactoryBase {
	DISALLOW_COPY_AND_ASSIGN(DefaultProjectFactory)
public:
	_NOUNUSED virtual IProject *createProject(const std::string &id,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual MainSegment *createMainSegment(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Include *createInclude(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ObjectsSegment *createObjectsSegment(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ProceduresSegment *createProceduresSegment(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Signature *createSignature(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual VirtualGroup *createVirtualGroup(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ConditionBlock *createConditionBlock(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Procedure *createProcedure(const std::string &id,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual StdLibCall *createStdLibCall(IType *libL,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ProcCall *createProcCall(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual NamedParam *createNamedParam(const std::string &typeName,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ReferenceParam *createProcParam(const std::string &ref,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual FloatParam *createProcParam(float f,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual IParam *createProcParam(const IParam *,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ProcInitStatement *createProcInitStatement(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Ellipse *createEllipse(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Point *createPoint(bool markerPoint, int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual IPoint *createPoint(const std::string &id, const IPoint &p, bool markerPoint,
										  int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Line *createLine(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Line *createLine(IPoint *p1, IPoint *p2, int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Curve *createCurve(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Poly *createPoly(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Fill *createFill(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Text *createText(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Vector2D *createVector2D(float x, float y,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual DebugProcedure *createDebugProcedure(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual RenderProcedure *createRenderProcedure(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual RenderCallback *createRenderCallback(const std::string &ref,
			IType *stdlib, int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual StringParam *createStringParam(const std::string &txt,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ParamSection *createParamSection(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual AssignmentOperator *createAssignmentOperator(const std::string &id,
			IType *typeL, int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual ExpressionParam *createExpressionParam(const std::string &,
			const IProjectFactory *, int line = -1) const throw(Exception::ReferenceException,
					Exception::ProjectFactoryException);

	_NOUNUSED virtual ExpressionParam *createExpressionParam(const ExpressionParam *,
			const IProjectFactory *, int line = -1) const throw(Exception::ReferenceException,
					Exception::ProjectFactoryException);

	_NOUNUSED virtual IType *createNullType(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Condition *createCondition(const HGL::Type::ConditionBase::OP &op,
			int line = -1) const throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual IfElse *createIfElse(int line = -1) const
	throw(Exception::ProjectFactoryException);

	_NOUNUSED virtual Pen *createPen(int line = -1) const throw(Exception::ProjectFactoryException);

protected:
	DefaultProjectFactory();

	template<class Factory>
	static inline typename Factory::point_type *createPoint(const Factory *const f,
			const std::string &id, const HGL::Type::IPoint &p, bool markerPoint, int line = -1)
	throw(Exception::ProjectFactoryException) {

		typename Factory::point_type *pt = f->createPoint(markerPoint, line);
		pt->attachType(f->createVector2D(p.getX(), p.getY(), line), false);
		pt->setId(id);
		return pt;
	}
};

}

}

#endif /* HGL_TYPE_DEFAULTPROJECTFACTORY_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
