/*
    Copyright (c) 2014, Heiko Schäfer <heiko@rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file eff_map.h
 * @brief An efficient std::map adder and updater based on Scott Meyer's Effective STL
 * @author Heiko Schäfer <heiko@rangun.de4>
 * @copyright BSD License
 */

#ifndef COMMONS_EFF_MAP_H
#define COMMONS_EFF_MAP_H

#include <map>

namespace Commons {

template<typename MapType, typename KeyArgType, typename ValueArgType>
typename MapType::iterator efficientAddOrUpdate(MapType &m, const KeyArgType &k,
		const ValueArgType &v, const typename MapType::iterator &lb) {

	if(lb != m.end() && !(m.key_comp()(k, lb->first))) {
		lb->second = v;
		return lb;
	} else {
		typedef typename MapType::value_type MVT;
		return m.insert(lb, MVT(k, v));
	}
}

template<typename MapType, typename KeyArgType, typename ValueArgType>
typename MapType::iterator efficientAddOrUpdate(MapType &m, const KeyArgType &k,
		const ValueArgType &v) {
	return efficientAddOrUpdate(m, k, v, m.lower_bound(k));
}

template<typename MapType>
struct key {
	typename MapType::value_type::first_type &operator()
	(const typename MapType::value_type &v) const {
		return v.first;
	}
};

template<typename MapType>
struct value {
	typename MapType::value_type::second_type &operator()
	(const typename MapType::value_type &v) const {
		return v.second;
	}
};

}

#endif /* COMMONS_EFF_MAP_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
