/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_EXPRESSIONPARAM_H
#define HGL_EXPRESSIONPARAM_H

#include "referenceparam.h"

class EvaluationVisitorTest;
class ExpressionParamTest;

namespace HGL {

namespace Interpreter {
class EvaluationVisitor;
class BinParser;
}

namespace Type {

class IProjectFactory;
class ExpressionParamImpl;

/**
 * @ingroup types
 * @ingroup params
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT ExpressionParam : public ReferenceParam {
	DISALLOW_COPY_AND_ASSIGN(ExpressionParam)

	template<class T> friend IType::Ptr::operator T *() const throw();
	friend class ::EvaluationVisitorTest;
	friend class ::ExpressionParamTest;
	friend class Interpreter::EvaluationVisitor;
	friend class Interpreter::BinParser;

public:
	using ReferenceHolderBase<TypeBase>::findReference;
	using IParam::findReference;
	using TypeBase::operator==;
	using IParam::operator==;
	using IParam::operator*;

	ACCEPT(ExpressionParam);

	virtual ~ExpressionParam();

	virtual const SerialID &getSerialID() const throw() _CONST;

	virtual const std::string toString() const;

	virtual bool operator==(const ExpressionParam &) const;
	virtual bool operator==(const IType &) const throw(Exception::InvalidExpressionException);

	virtual ExpressionParam &operator=(const IType &) throw(Exception::InvalidExpressionException);

	virtual IType *operator+(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator-(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator*(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator/(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *power(const IType &) throw(Exception::InvalidExpressionException);

	virtual IType *sinus() throw(Exception::InvalidExpressionException);
	virtual IType *cosinus() throw(Exception::InvalidExpressionException);
	virtual IType *operator-() throw(Exception::InvalidExpressionException);

	_INTERNAL _NOUNUSED ExpressionParam *addLeftOperand(IParam *ip,
			OP op) const throw(Exception::InvalidExpressionException) _NONNULL(2);

	OP getOperator() const _PURE;
	void setOperator(const OP &);

	bool isUnary() const _PURE;
	bool isPrefixed() const _PURE;

	virtual std::string getTypeName() const throw();

	virtual const IType *findReference(bool recache = false) const;

protected:
	ExpressionParam(const std::string &ref,
					const IProjectFactory *fac) throw(Exception::ReferenceException);
	ExpressionParam(const ExpressionParam *ep,
					const IProjectFactory *fac) throw(Exception::AttachException,
							Exception::ReferenceException);

	virtual const SerialID *const *getAllowedTypes() const throw();
	void setPrefixed(bool b);

private:
	class ExpressionParamImpl *const _pimpl;
};

}

template<> inline _HOT IType::Ptr::operator Type::ExpressionParam *() const throw() {
	return reinterpret_cast<Type::ExpressionParam *>(m_ptr);
}

extern template IType::Ptr::operator Type::ExpressionParam *() const throw();

}

#endif // HGL_EXPRESSIONPARAM_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
