/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_FLOATPARAM_H
#define HGL_TYPE_FLOATPARAM_H

#include "raiifactory.h"
#include "parambase.h"

#if CPPUNIT
class EvaluationVisitorTest;
class ExpressionParamTest;
#endif

namespace HGL {

namespace ControlSource {
class ParameterFactory;
}

namespace Type {

/**
 * @ingroup types
 * @ingroup params
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT FloatParam : public ParamBase {
	DISALLOW_COPY_AND_ASSIGN(FloatParam)

	friend class HGL::ControlSource::ParameterFactory;

#if CPPUNIT
	friend class ::ExpressionParamTest;
	friend class ::EvaluationVisitorTest;
#endif

public:
	using ParamBase::operator==;
	using IParam::operator*;

	ACCEPT(FloatParam);

	virtual const std::string toString() const;

	float getFloat() const _PURE;

	virtual bool operator==(const IType &) const throw(Exception::InvalidExpressionException);
	virtual bool operator<(const IType &) const throw(Exception::InvalidExpressionException);

	virtual operator float() const throw(Exception::ReferenceException,
										 Exception::InvalidExpressionException) _PURE;
	virtual operator Type::Vector2D &() const throw(Exception::InvalidExpressionException) _PURE;
	virtual operator Type::StringParam &() const throw(Exception::InvalidExpressionException);

	IType &operator=(const IType &) throw(Exception::InvalidExpressionException);

	IType *operator+(const IType &) throw(Exception::ReferenceException,
										  Exception::InvalidExpressionException);
	IType *operator-(const IType &) throw(Exception::ReferenceException,
										  Exception::InvalidExpressionException);
	IType *operator*(const IType &) throw(Exception::ReferenceException,
										  Exception::InvalidExpressionException);
	IType *operator/(const IType &) throw(Exception::ReferenceException,
										  Exception::InvalidExpressionException);
	FloatParam *operator-() throw(Exception::InvalidExpressionException);

	IType *power(const IType &) throw(Exception::InvalidExpressionException);
	FloatParam *sinus() throw(Exception::InvalidExpressionException);
	FloatParam *cosinus() throw(Exception::InvalidExpressionException);

	virtual IType::Ptr clone() const throw(Exception::CloneNotSupportedException);

protected:
	explicit FloatParam(float f);
	~FloatParam();

	const SerialID &getSerialID() const throw() _CONST;

private:
	_INTERNAL IType *floatOp(const IType &, OP) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);

	_INTERNAL FloatParam *performOp(float f, OP) throw(Exception::InvalidExpressionException);
	_INTERNAL IType *performOp(Vector2D &v, OP) throw(Exception::InvalidExpressionException);

private:
	Common::RAIIFactory<Vector2D> m_vraf;

	float m_float;
	Vector2D *m_vector2D;
};

}

template<> inline _HOT IType::Ptr::operator Type::FloatParam *() const throw() {
	return reinterpret_cast<Type::FloatParam *>(m_ptr);
}

extern template IType::Ptr::operator Type::FloatParam *() const throw();

namespace Common {
	extern template class RAIIFactory<HGL::Type::Vector2D>;
}

}

#endif /* HGL_TYPE_FLOATPARAM */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
