/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_COMMON_FLOATUTIL_H
#define HGL_COMMON_FLOATUTIL_H

#include <cmath>

#include "linkercontrol.h"

#ifndef FLOATLESSEQUAL
#if _XOPEN_SOURCE >= 600 || _ISOC99_SOURCE || _POSIX_C_SOURCE >= 200112L
#define FLOATLESSEQUAL(a, b) std::islessequal((a), (b))
#else
#define FLOATLESSEQUAL(a, b) ((a) <= (b))
#endif
#endif

namespace HGL {

namespace Common {

class FloatUtil {
	DISALLOW_COPY_AND_ASSIGN(FloatUtil)
public:
	_EXPORT virtual ~FloatUtil();

	_EXPORT static std::string formatFloat(float f);

	_EXPORT static bool nequalFloat(float a, float b) _PURE;

	inline static bool equalFloat(float a, float b) {
		return FLOATLESSEQUAL(fabsf(a - b), (m_epsilon * std::max(fabsf(a), fabsf(b))));
	}

protected:
	_EXPORT FloatUtil();

private:
	static const float m_epsilon _EXPORT;
};

}

}

#endif /* HGL_COMMON_FLOATUTIL_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
