/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_GPGME_CONTEXT
#define HGL_COMMON_GPGME_CONTEXT

#include "gpgmeexception.h"
#include "l10n.h"

namespace HGL {

namespace Common {

namespace GPGME {

class _INTERNAL Context {
	DISALLOW_COPY_AND_ASSIGN(Context)
public:
	inline Context() throw(GPGMEException) : m_ctx(NULL) {
		gpgme_error_t err = gpgme_new(&m_ctx);

		if(unlikely(err != GPG_ERR_NO_ERROR)) {
			throw GPGMEException(_("Creating GPGME context failed"), err);
		}
	}

	inline ~Context() throw() {
		gpgme_release(m_ctx);
	}

	inline operator gpgme_ctx_t() const throw() {
		return m_ctx;
	}

private:
	gpgme_ctx_t m_ctx;
};

}

}

}

#endif /* HGL_COMMON_GPGME_CONTEXT */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
