/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_GPGME_GPGMEEXCEPTION
#define HGL_COMMON_GPGME_GPGMEEXCEPTION

#include <string>

#if defined(ENABLE_SIGNING) || defined(IN_KDEVELOP_PARSER)
#include <gpgme.h>
#endif

#include "hglexception.h"

namespace HGL {

namespace Common {

namespace GPGME {

/**
 * @ingroup exceptions
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT GPGMEException : public HGL::Exception::HGLException {
	GPGMEException &operator=(const GPGMEException &);
public:

#if ENABLE_SIGNING
	GPGMEException(const std::string &msg, gpgme_error_t err);
#else
	GPGMEException(const std::string &msg, unsigned int err);
#endif

	virtual ~GPGMEException() throw();

	virtual const char *what() const throw();

private:
#if ENABLE_SIGNING
	gpgme_error_t m_err;
#endif
	std::string m_msg;
};

}

}

}

#endif /* HGL_COMMON_GPGME_GPGMEEXCEPTION */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
