/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_LUA_ILUATYPE_H
#define HGL_TYPE_LUA_ILUATYPE_H

#include <string>

#include <stdint.h>

extern "C" {
#include <lua.h>
}

#include "linkercontrol.h"

namespace HGL {

namespace Common {
class IApplication;
class URI;
}

namespace Type {
/**
 * @interface ILuaType
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT ILuaType {
	DISALLOW_COPY_AND_ASSIGN(ILuaType)
public:
	typedef struct {
		lua_CFunction closure;
		const char *const name;
		const bool hasILuaType;
	} LUAFUNCINFO;

	_INTERNAL virtual ~ILuaType() {}

	_INTERNAL virtual const std::string &getId() const = 0;
	_INTERNAL virtual const std::string &getProjectName() const = 0;

	_INTERNAL virtual const Common::IApplication *getApplication() const = 0;
	_INTERNAL virtual void setApplication(const Common::IApplication *const app) = 0;

	_INTERNAL virtual void setLuaInclude(const Common::URI &file) = 0;
	_INTERNAL virtual void setLuaScript(const char *const , bool = false) = 0;

	_INTERNAL virtual const char *getLuaScript() const = 0;
	_INTERNAL virtual uint32_t getLuaScriptLength() const = 0;

protected:
	_INTERNAL inline ILuaType() {}
};

}

}

#endif // HGL_TYPE_LUA_ILUATYPE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
