/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_IMAGEOUTPUT_IMAGEOUTPUTEXCEPTION_H
#define HGL_IMAGEOUTPUT_IMAGEOUTPUTEXCEPTION_H

#include "hglexception.h"

namespace HGL {

namespace ImageOutput {

/**
 * @ingroup exceptions
 * @ingroup ioplugins
 * @brief Exception thrown if there has been an unrecoverable error
 * @relates HGL::ImageOutput::IOutput
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT ImageOutputException : public HGL::Exception::HGLException {
	ImageOutputException &operator=(const ImageOutputException &);
public:
	/**
	 * @brief Contructor
	 *
	 * @param msg message of the error
	 * @param prefix an optional prefix to the message
	 **/
	ImageOutputException(const std::string &msg, const std::string &prefix = std::string());
	virtual ~ImageOutputException() throw();

	virtual const char *what() const throw() _PURE;

private:
	std::string m_msg;
};

}

}

#endif // HGL_IMAGEOUTPUT_CANVASEXCEPTION_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
