/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_INVALIDEXPRESSIONEXCEPTION_H
#define HGL_INVALIDEXPRESSIONEXCEPTION_H

#include "parserexception.h"

namespace HGL {

class IType;

namespace Exception {

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
/**
 * @ingroup exceptions
 * @brief Exception if an expression is invalid
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT InvalidExpressionException : public Exception::ParserException {
	InvalidExpressionException &operator=(const InvalidExpressionException &);
public:
	InvalidExpressionException(const std::string &msg, const IType *op = 0);
	virtual ~InvalidExpressionException() throw();

	virtual const char *what() const throw() _PURE;

	virtual const IType *getOperand() const throw() _PURE;

	virtual void setLine(int line);

private:
	const IType *m_op;
	std::string m_myMessage;
};
#pragma GCC diagnostic pop

}

}

#endif // HGL_INVALIDEXPRESSIONEXCEPTION_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
