/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file ipluginpoptoptions.h
 * @brief Interface to enable command line options for plugins
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.62
 **/

#ifndef HGL_IPLUGINPOPTOPTIONS_H
#define HGL_IPLUGINPOPTOPTIONS_H

#include "linkercontrol.h"

struct poptOption;

namespace HGL {

namespace Common {

/**
 * @defgroup popt Command line options support
 * @brief Command line options support
 *
 * See http://linux.die.net/man/3/popt for how to create and use the popt structure
 *
 * @todo Write better documentation on how to enable command line support
 */

/**
 * @ingroup popt
 * @interface IPluginPoptOptions
 * @brief Interface to enable command line options for plugins
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.62
 *
 **/
class _EXPORT IPluginPoptOptions {
	IPluginPoptOptions(const IPluginPoptOptions &);
	IPluginPoptOptions &operator=(const IPluginPoptOptions &);
public:
	virtual ~IPluginPoptOptions() = 0;

	/**
	 * @name Command line options support
	 * @{
	 */

	/**
	 * @brief Get plugin supplied command line options
	 *
	 * @note See http://linux.die.net/man/3/popt for how to create and use the popt structure and
	 * @ref popt for details on how to use it in your plugins.
	 *
	 * The default implementation returns <code>NULL</code> i.e. it won't provide own command line
	 * options
	 *
	 * In your @c POPT_ARG_CALLBACK you can indicate HGL to exit gracefully by throwing
	 * HGL::Exception::PoptExitException
	 *
	 * @return structure to popt options
	 * @since 0.4.53
	 **/
	_NOUNUSED virtual struct ::poptOption *getPoptOptions() const _CONST;

	//@}

protected:
	IPluginPoptOptions();
};

}

}

#endif // HGL_IPLUGINPOPTOPTIONS_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
