/*
    Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_IPOINT_H
#define HGL_TYPE_IPOINT_H

#include "colorbase.h"
#include "ireferenceholder.h"

namespace HGL {

namespace Type {

/**
 * @ingroup types
 * @interface IPoint
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT IPoint : public virtual IReferenceHolder, public ColorBase {
	DISALLOW_COPY_AND_ASSIGN(IPoint)
public:
	using IType::operator=;

	virtual ~IPoint();

	virtual float getX() const throw(Exception::ReferenceException) = 0;
	virtual float getY() const throw(Exception::ReferenceException) = 0;

	virtual Vector2D *getVector2D() const throw(Exception::ReferenceException) = 0;

	virtual float getDistance(const IPoint &p) const throw(Exception::ReferenceException) = 0;

	virtual bool isMarkerPoint() const = 0;

	virtual bool operator==(const IPoint &p) const throw(Exception::ReferenceException) = 0;

	friend bool operator==(const IPoint &,
						   const BaseVector2D &) throw(Exception::ReferenceException);

protected:
	inline IPoint() : ColorBase() {}
};

}

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
