/*
    Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_LINE_H
#define HGL_TYPE_LINE_H

#include "proxy.h"

namespace HGL {

namespace Type {

class IPoint;
class Point;

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED Line : public Utility::Proxy<Line> {
	DISALLOW_COPY_AND_ASSIGN(Line)
	friend class ProjectFactoryBase;
public:
	using Utility::Proxy<Line>::base::findReference;
	using IType::operator=;
	using IType::attachType;
	using TypeBase::operator*;

	ACCEPT(Line);

	~Line();

	_INTERNAL float getM() const;
	_INTERNAL float getB() const;

	_INTERNAL bool isOnLine(const Type::IPoint &p) const;

// 	_INTERNAL const Type::IPoint *getMedian() const;

	enum _INTERNAL _PACKED e_point { P1, P2 };
	_NOUNUSED virtual const Type::IPoint *getPoint(e_point p) const;

	virtual Line *operator+(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual Line *operator*(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual Line *operator*(const float *) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) _NONNULL(2);

	virtual BOUNDINGBOX getBoundingBox() const;

	const Type::SerialID &getSerialID() const throw() _CONST;

protected:
	Line();
	Line(Type::IPoint *p1, Type::IPoint *p2);

	virtual IType *attachType(IType *type, const TYPELIST::iterator &before,
							  bool check) throw(Exception::AttachException) _NONNULL(2);

	_NOUNUSED virtual const IType *findReference(bool recache = false) const;

	virtual const Type::SerialID *const *getAllowedTypes() const throw();

	virtual void referenceHasChanged(const std::string &old, const std::string &now, bool removed);

private:
	Point *m_pt[2];
	mutable BOUNDINGBOX *m_bb;
};

template<> struct IReferenceHolder::PropertyTraits<Line> {
	typedef Line type;
	bool operator()(const type *, const type *) const throw();
};

extern template bool IReferenceHolder::isPropertyEqual<Line>
(const IReferenceHolder::PropertyTraits<Line>::type *) const throw();

extern template class Utility::Proxy<Line>;

}

template<> inline _HOT IType::Ptr::operator Type::Line *() const throw() {
	return reinterpret_cast<Type::Line *>(m_ptr);
}

extern template IType::Ptr::operator Type::Line *() const throw();

}

#endif /* HGL_TYPE_LINE_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
