/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_LOGGER_H
#define HGL_COMMON_LOGGER_H

#define LOGGER_CLASS HGL::Common::Logger
#define LOGGER_PREFIX L""

#include "basiclogger.h"

#undef logDebug

#ifndef NDEBUG
#define logDebug(msg)	{ (LOGGER_CLASS(LOGGER_CLASS::LOG_DEBUG)) << __FUNCTION__ << ": " << msg; }
#else
#define logDebug(msg)	{}
#endif

namespace HGL {

class IType;

namespace Type {
class IParam;
class SerialID;
class ReferenceParam;
class ExpressionParam;
}

namespace Common {

typedef Commons::BasicLogger < std::ostreambuf_iterator<wchar_t> > WCLOG_BASELOGGER;

/**
 * @ingroup util
 * @brief Console logger (to @c std::wclog)
 *
 * @note This logger cannot be used in static initialization routines!
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _PACKED _EXPORT Logger : public WCLOG_BASELOGGER {
	DISALLOW_COPY_AND_ASSIGN(Logger)
public:
	using WCLOG_BASELOGGER::operator<<;

	Logger(const LEVEL &lvl);
	virtual ~Logger();

	typedef struct _noconv {} noconv;

	friend WCLOG_BASELOGGER &operator<<(WCLOG_BASELOGGER &l, const noconv &);

	virtual WCLOG_BASELOGGER &operator<<(const std::exception &e);
	virtual WCLOG_BASELOGGER &operator<<(const std::string &s);
	virtual WCLOG_BASELOGGER &operator<<(const char *s);

private:
	bool m_noconv;
};

}

}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wredundant-decls"
namespace Commons {
template<> const char *HGL::Common::WCLOG_BASELOGGER::getLevelString(LEVEL lvl) const;
}
#pragma GCC diagnostic pop

namespace HGL {

_EXPORT extern Common::WCLOG_BASELOGGER &operator<<(Common::WCLOG_BASELOGGER &l,
		const HGL::Type::IParam *ip);

_EXPORT extern Common::WCLOG_BASELOGGER &operator<<(Common::WCLOG_BASELOGGER &l,
		const HGL::Type::ReferenceParam *rp);

_EXPORT extern Common::WCLOG_BASELOGGER &operator<<(Common::WCLOG_BASELOGGER &l,
		const HGL::Type::ExpressionParam *ep);

_EXPORT extern Common::WCLOG_BASELOGGER &operator<<(Common::WCLOG_BASELOGGER &l,
		const HGL::IType *type);

_EXPORT extern Common::WCLOG_BASELOGGER &operator<<(Common::WCLOG_BASELOGGER &l,
		const HGL::Type::SerialID &serialID);
}

extern template class Commons::BasicLogger<std::ostreambuf_iterator<wchar_t> >;

#endif /* HGL_COMMON_LOGGER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
