/*
    Copyright (c) 2014, Heiko Schäfer <heiko@rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_COMMON_LOOKUP_MAP_H
#define HGL_COMMON_LOOKUP_MAP_H

#include "util.h"
#include "eff_map.h"

namespace HGL {

namespace Common {

/**
 * @ingroup util
 * @brief Looks up a value in a lookup map and if neccessary creates the value first
 *
 * @tparam MapType the type of the lookup map
 * @tparam KeyArgType they type of the lookup key
 * @tparam ValueFunction a function to create a value if not already in the lookup map
 *
 * @param m the lookup map
 * @param k the key to look up
 * @param v a value creating function which gets called @b only if the value isn't yet in the map
 *
 * @return the looked up value
 */
template<typename MapType, typename KeyArgType, typename ValueFunction>
typename MapType::value_type::second_type lookupInsert(MapType &m, const KeyArgType &k,
		const ValueFunction &v) {

	const typename MapType::iterator &li(m.lower_bound(k));

	if((li != m.end() && !(m.key_comp()(k, li->first)))) {
		return li->second;
	} else {
		typedef typename MapType::value_type MVT;
		return m.insert(li, MVT(k, v()))->second;
	}
}

}

}

#endif /* HGL_COMMON_LOOKUP_MAP_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
