/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_LUA_LUATYPEBASE_H
#define HGL_TYPE_LUA_LUATYPEBASE_H

#include <set>

#include "iluatype.h"

namespace HGL {
class IType;
}

_EXPORT extern const char *argumentError(lua_State *l, int minArgs) _NONNULL(1);
_EXPORT extern bool registerCurrentLuaType(lua_State *l, HGL::Type::ILuaType *cur,
		int envidx) _NONNULL(1, 2);
_EXPORT extern void removeLuaFuncs(lua_State *l, int envidx) _NONNULL(1);
_EXPORT extern HGL::IType *getCurrentType(lua_State *l) _NONNULL(1);

_EXPORT _NOUNUSED extern HGL::IType *getLuaIType(lua_State *l, int argc,
		const char **id) _NONNULL(1, 3);

_EXPORT extern HGL::IType *resolveFast(HGL::IType *t, const std::string &id) _NONNULL(1);

namespace HGL {

#if INTERPRETER
namespace Interpreter {
class ExecuteVisitor;
class EvaluationVisitor;
}
#endif

namespace Compiler {
class LuaCompilerBase;
}

namespace Type {

/**
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT LuaTypeBase : public ILuaType {
	DISALLOW_COPY_AND_ASSIGN(LuaTypeBase)

#if INTERPRETER
	friend class Interpreter::EvaluationVisitor;
	friend class Interpreter::ExecuteVisitor;
#endif

	friend class Compiler::LuaCompilerBase;

public:
	virtual const char *getLuaScript() const _PURE;
	virtual void setLuaScript(const char *const , bool = false) _NONNULL(2);
	virtual void setLuaInclude(const Common::URI &file);

	void setLuaScriptLength(uint32_t);

	static bool registerLuaFuncs(lua_State *l, ILuaType *ilt, const LUAFUNCINFO *const lfi,
								 size_t n, int envidx);
	static void unregisterLuaFuncs(lua_State *l, const LUAFUNCINFO *const lfi, int envidx);

protected:
	LuaTypeBase();
	virtual ~LuaTypeBase();

	virtual uint32_t getLuaScriptLength() const _PURE;

private:
	char *m_luaScript;
	size_t m_luaScriptLen;

	static _WEAK class _luaInit {
		DISALLOW_COPY_AND_ASSIGN(_luaInit)
	public:
		_EXPORT _luaInit() _COLD;
		_EXPORT ~_luaInit() _COLD;

		lua_State *m_luaState;
	} m_luaInit;
};

}

}

#endif // HGL_TYPE_LUA_LUATYPEBASE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
