/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_PARAMBASE_H
#define HGL_TYPE_PARAMBASE_H

#include "typebase.h"
#include "iparam.h"

namespace HGL {

namespace Type {

class StringParam;

/**
 * @ingroup types
 * @ingroup params
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT _PACKED ParamBase : public virtual IParam, public virtual TypeBase {
	DISALLOW_COPY_AND_ASSIGN(ParamBase)
public:
	using IType::operator=;
	using IParam::operator==;
	using TypeBase::operator-;
	using TypeBase::operator*;
	using TypeBase::operator==;

	virtual const std::string toString() const = 0;

	virtual bool operator==(const IParam &) const;
	virtual bool operator==(const IType &) const throw(Exception::InvalidExpressionException);

	virtual HGL::IType *operator+(const IType &op) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual HGL::IType *operator-(const IType &op) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual HGL::IType *operator*(const IType &op) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual HGL::IType *operator/(const IType &op) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual HGL::IType *power(const IType &op) throw(Exception::InvalidExpressionException);

	virtual HGL::IType::BOUNDINGBOX getBoundingBox() const;

protected:
	_INTERNAL ParamBase();
	_INTERNAL explicit ParamBase(const std::string &id);
	virtual ~ParamBase();

	const IType *findReference(const IType *) const _NONNULL(2);

	virtual const SerialID *const *getAllowedTypes() const throw();

protected:
	mutable Type::StringParam *m_toSP;
};

extern template HGL::IType *HGL::Type::IParam::operate < HGL::IType *( *)(HGL::Type::IParam *,
		HGL::IType const *), HGL::IType *( *)(HGL::Type::TypeBase *, HGL::IType const &) >
(HGL::IType const &, HGL::IType * ( *)(HGL::Type::IParam *, HGL::IType const *),
 HGL::IType * ( *)(HGL::Type::TypeBase *, HGL::IType const &));

}

}

#endif // HGL_TYPE_PARAMBASE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
