/*
    Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_POINT_H
#define HGL_TYPE_POINT_H

#include "ipoint.h"
#include "proxy.h"

class LuaProcedure;

namespace HGL {

namespace Type {

class IProjectFactory;
class Vector2D;

_EXPORT bool operator==(const IPoint &, const BaseVector2D &) throw(Exception::ReferenceException);

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED Point : public virtual Type::IPoint, public Utility::Proxy<Point> {
	DISALLOW_COPY_AND_ASSIGN(Point)

	friend class ProjectFactoryBase;
	friend class ::LuaProcedure;
	friend class Line;

public:
	using ColorBase::setColor;
	using TypeBase::operator*;
	using TypeBase::operator==;
	using TypeBase::attachType;
	using IType::operator-;

	ACCEPT(Point);

	~Point();

	virtual uint32_t getColorInteger() const;
	virtual void setColor(uint32_t intColor);

	virtual float getX() const throw(Exception::ReferenceException) _HOT;
	virtual float getY() const throw(Exception::ReferenceException) _HOT;

	virtual float getDistance(const IPoint &) const throw(Exception::ReferenceException);

	virtual bool isMarkerPoint() const;

	virtual bool operator==(const IPoint &) const throw(Exception::ReferenceException);

	virtual Point &operator=(const IType &) throw(Exception::InvalidExpressionException);

	virtual IType *operator+(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator-(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator*(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *operator*(const float *) throw(Exception::InvalidExpressionException) _NONNULL(2);
	virtual IType *operator/(const IType &) throw(Exception::InvalidExpressionException);

	virtual operator Vector2D &() const throw(Exception::InvalidExpressionException);
	virtual Vector2D *getVector2D() const throw(Exception::ReferenceException) _HOT;

	virtual BOUNDINGBOX getBoundingBox() const;

	_INTERNAL virtual bool isValid() const throw() {
		const TYPELIST &tl(getAttachedTypes());
		return Utility::Proxy<Point>::base::isValid() || (!tl.empty() &&
				tl.front() == SerialID::VECTOR2D);
	}

	virtual const SerialID &getSerialID() const throw() _CONST;

protected:
	explicit Point(bool markerPoint);

	virtual const SerialID *const *getAllowedTypes() const throw();

private:
	bool m_markerPoint;
	mutable Vector2D *m_v2d;
};

template<> struct IReferenceHolder::PropertyTraits<Point> {
	typedef Point type;
	bool operator()(const type *, const type *) const throw();
};

extern template bool IReferenceHolder::isPropertyEqual<Point>
(const IReferenceHolder::PropertyTraits<Point>::type *) const throw();

extern template class Utility::Proxy<Point>;

}

template<> inline _HOT IType::Ptr::operator Type::Point *() const throw() {
	return reinterpret_cast<Type::Point *>(m_ptr);
}

extern template IType::Ptr::operator Type::Point *() const throw();

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
