/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file poptexitexception.h
 * @brief Exception to indicate that HGL has to successfully terminate after processing an
 * command line argument.
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.65
 **/

#ifndef HGL_POPTEXITEXCEPTION_H
#define HGL_POPTEXITEXCEPTION_H

#include <cstdlib>

#include "hglexception.h"

namespace HGL {

namespace Exception {

/**
 * @defgroup exceptions Exceptions
 * @if PLUGIN_API
 * @brief Exceptions for use in plugins
 *
 * If you use other exceptions, please take care they are all handled properly.
 * @else
 * @brief Overview of all exceptions
 * @endif
 */

/**
 * @ingroup exceptions
 * @ingroup popt
 * @brief Exception to indicate that HGL has to successfully terminate after processing an
 * command line argument.
 *
 * Some command line arguments show only information and than want to exit. For this case
 * they should throw this exception to indicate HGL it can terminate gracefully without
 * doing further action.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.65
 **/
class _EXPORT PoptExitException : public HGLException {
	PoptExitException &operator=(const PoptExitException &);
public:
	/**
	 * @brief Constructs an @c PoptExitException
	 * @param ret the return value with which the application should exit
	 * @since 0.5.19
	 */
	PoptExitException(int ret = EXIT_SUCCESS) throw();

	/**
	 * @brief Gets the return value with which the application should exit
	 * @param ret the return value with which the application should exit
	 * @since 0.5.19
	 */
	int getReturnCode() const throw() _PURE;

private:
	int m_ret;
};

}

}

#endif // HGL_POPTEXITEXCEPTION_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
