/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGLPROCEDURE_H
#define HGLPROCEDURE_H

#include "typebase.h"
#include "luatypebase.h"

namespace HGL {

namespace Type {

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED  Procedure : public TypeBase, public LuaTypeBase {
	DISALLOW_COPY_AND_ASSIGN(Procedure)
public:
	using TypeBase::attachType;

	ACCEPT(Procedure);

	using IType::operator=;

	virtual const std::string &getId() const _PURE;

protected:
	Procedure(const std::string &id);
	~Procedure();

	const SerialID &getSerialID() const throw() _CONST;

	virtual const SerialID *const *getAllowedTypes() const throw();

	virtual IType *attachType(IType *type, const TYPELIST::iterator &before,
							  bool check = true) throw(Exception::AttachException) _NONNULL(2);

private:
	static bool notParamSectionNotProcInit(const IType *t) throw();
};

}

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
