/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_PROJECT_H
#define HGL_TYPE_PROJECT_H

#include "nonemptytype.h"
#include "iproject.h"

namespace HGL {

namespace Type {

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED Project : public virtual IProject, public NonEmptyType {
	DISALLOW_COPY_AND_ASSIGN(Project)

	friend class ProjectFactoryBase;

public:
	ACCEPT(Project);

	using IType::operator=;

	virtual MainSegment *getMainSegment() const _PURE;
	virtual ObjectsSegment *getObjectsSegment() const _PURE;
	virtual ProceduresSegment *getProceduresSegment() const _PURE;

	_NOUNUSED virtual MainSegment *addMainSegment(MainSegment *ms);
	_NOUNUSED virtual ObjectsSegment *addObjectsSegment(ObjectsSegment *os);
	_NOUNUSED virtual ProceduresSegment *addProceduresSegment(ProceduresSegment *ps);

	/**
	 * A Project is @b valid if it at least contains a MainSegment.
	 * @copydoc NonEmptyType::isValid
	 **/
	virtual bool isValid() const throw();

	const SerialID &getSerialID() const throw() _CONST;

protected:
	Project(const std::string &id);
	virtual ~Project();

	virtual const SerialID *const *getAllowedTypes() const throw();

private:
	static bool isMainSegment(HGL::IType *t) throw() __attribute__((const));

private:
	MainSegment *m_mainSegment;
	ObjectsSegment *m_objectsSegment;
	ProceduresSegment *m_proceduresSegment;
};

}

template<> inline _HOT IType::Ptr::operator Type::Project *() const throw() {
	return reinterpret_cast<Type::Project *>(m_ptr);
}

extern template IType::Ptr::operator Type::Project *() const throw();

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
