/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_PROJECTFACTORYBASE_H
#define HGL_TYPE_PROJECTFACTORYBASE_H

#include "raiifactory.h"
#include "iprojectfactory.h"

namespace HGL {

namespace Type {

/**
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _INTERNAL ProjectFactoryBase : public IProjectFactory {
	DISALLOW_COPY_AND_ASSIGN(ProjectFactoryBase)
protected:
	inline ProjectFactoryBase() : IProjectFactory(), m_raiiFactory() {}
	_EXPORT virtual ~ProjectFactoryBase() = 0;

	inline const Common::RAIIFactory<IType> &getRAIIFactory() const {
		return m_raiiFactory;
	}

	template<class TypeClass>
	TypeClass *createType() const throw(Exception::HGLException);

	template<class TypeClass, typename P1>
	TypeClass *createType(const P1 &) const throw(Exception::HGLException);

	template<class TypeClass, typename P1, typename P2>
	TypeClass *createType(const P1 &, const P2 &) const throw(Exception::HGLException);

	template<class TypeClass, typename P1, typename P2, typename P3>
	TypeClass *createType(const P1 &, const P2 &, const P3 &) const throw(Exception::HGLException);

	template<class TypeClass, typename P1, typename P2, typename P3, typename P4>
	TypeClass *createType(const P1 &, const P2 &, const P3 &,
						  const P4 &) const throw(Exception::HGLException);

private:
	template<class TypeClass>
	TypeClass *check(TypeClass *const t) const throw(Exception::ProjectFactoryException);

private:
	Common::RAIIFactory<IType> m_raiiFactory;
};

template<class TypeClass>
inline TypeClass *
ProjectFactoryBase::check(TypeClass *const t) const throw(Exception::ProjectFactoryException) {
	if(likely(t != 0)) {
		return t;
	} else {
		throw Exception::ProjectFactoryException();
	}
}

template<class TypeClass>
inline TypeClass *
ProjectFactoryBase::createType() const throw(Exception::HGLException) {

	Common::Util::disableMemoryHandler();
	TypeClass *t = new(std::nothrow) TypeClass();
	Common::Util::enableMemoryHandler();

	return check(t);
}

template<class TypeClass, typename P1>
inline TypeClass *
ProjectFactoryBase::createType(const P1 &p1) const throw(Exception::HGLException) {

	Common::Util::disableMemoryHandler();
	TypeClass *t = new(std::nothrow) TypeClass(p1);
	Common::Util::enableMemoryHandler();

	return check(t);
}

template<class TypeClass, typename P1, typename P2>
inline TypeClass *
ProjectFactoryBase::createType(const P1 &p1, const P2 &p2) const throw(Exception::HGLException) {

	Common::Util::disableMemoryHandler();
	TypeClass *t = new(std::nothrow) TypeClass(p1, p2);
	Common::Util::enableMemoryHandler();

	return check(t);
}

template<class TypeClass, typename P1, typename P2, typename P3>
inline TypeClass *ProjectFactoryBase::createType(const P1 &p1, const P2 &p2, const P3 &p3) const
throw(Exception::HGLException) {

	Common::Util::disableMemoryHandler();
	TypeClass *t = new(std::nothrow) TypeClass(p1, p2, p3);
	Common::Util::enableMemoryHandler();

	return check(t);
}

template<class TypeClass, typename P1, typename P2, typename P3, typename P4>
inline TypeClass *ProjectFactoryBase::createType(const P1 &p1, const P2 &p2, const P3 &p3,
		const P4 &p4) const throw(Exception::HGLException) {

	Common::Util::disableMemoryHandler();
	TypeClass *t = new(std::nothrow) TypeClass(p1, p2, p3, p4);
	Common::Util::enableMemoryHandler();

	return check(t);
}

}

namespace Common {

extern template class RAIIFactory<HGL::IType>;

}

}

#endif // HGL_TYPE_PROJECTFACTORYBASE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
