/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_UTILITY_PROXY_H
#define HGL_TYPE_UTILITY_PROXY_H

#include "referenceholderbase.h"
#include "serialid.h"
#include "typeutil.h"
#include "logger.h"
#include "l10n.h"

namespace HGL {

namespace Type {

namespace Utility {

template < class T, class TBase = HGL::Type::TypeBase,
		 class Base = HGL::Type::ReferenceHolderBase<TBase> >
class _EXPORT Proxy : public Base {
	DISALLOW_COPY_AND_ASSIGN(Proxy)
public:
	typedef Base base;

	using HGL::IType::operator=;

	virtual bool operator==(const IType &t) const throw(Exception::InvalidExpressionException);

protected:
	Proxy(const SerialID &sid) throw();
	~Proxy();

	inline T *getTarget() const throw() {
		return const_cast<T *>(static_cast<const T *>(m_proxy == this ? this :
							   getProxy()->getTarget()));
	}

	virtual void referenceHasChanged(const std::string &old, const std::string &now, bool removed);

private:
	T *getProxy() const throw();

private:
	const SerialID &m_sid;
	mutable Proxy<T, TBase, Base> *m_proxy;
};

template<class T, class TBase, class Base>
Proxy<T, TBase, Base>::Proxy(const SerialID &sid) throw() : Base(), m_sid(sid), m_proxy(0L) {}

template<class T, class TBase, class Base>
Proxy<T, TBase, Base>::~Proxy() {}

template<class T, class TBase, class Base>
bool Proxy<T, TBase, Base>::operator==(const IType &t) const
throw(Exception::InvalidExpressionException) {
	typedef typename IReferenceHolder::PropertyTraits<T>::type type;
	return t == m_sid ?
		   Base::template isPropertyEqual<type>(dynamic_cast<const type *>(&t)) : false;
}

template<class T, class TBase, class Base> inline
void Proxy<T, TBase, Base>::referenceHasChanged(const std::string &, const std::string &, bool) {
	m_proxy = 0L;
}

template<class T, class TBase, class Base>
T *Proxy<T, TBase, Base>::getProxy() const throw() {

	if(!m_proxy) {

		m_proxy = static_cast<T *>(const_cast<Proxy<T, TBase, Base> *>(this));
		const IType *proxy;

		if(Base::getReference() && (proxy = Base::findReference())) {
			if(proxy->isNull()) {
				logWarning(Util::format(_("%s: could not find reference to \'%s\'"),
										m_sid.getHumanReadableName(),
										Base::getReference()->c_str()));
			} else if(proxy == m_sid
					  && (!(m_proxy = const_cast<T *>(reinterpret_cast<const T *>(proxy))))) {
				m_proxy = const_cast<Proxy<T, TBase, Base> *>(this);
			}
		}
	}

	return static_cast<T *>(m_proxy);
}

}

}

}

#endif /* HGL_TYPE_UTILITY_PROXY_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
