/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_REFERENCEHOLDERBASE_H
#define HGL_TYPE_REFERENCEHOLDERBASE_H

#include "referenceholdertbase.h"
#include "typebase.h"

namespace HGL {

namespace Type {

template<class Base>
class _EXPORT ReferenceHolderBase : public ReferenceHolderTBase, public virtual Base {
public:
	using HGL::IType::operator=;

	typedef Base base;

	virtual bool operator==(const IType &t) const throw(Exception::InvalidExpressionException);

protected:
	ReferenceHolderBase();
	explicit ReferenceHolderBase(IType *lib);
	_INTERNAL explicit ReferenceHolderBase(const std::string &ref)
	throw(HGL::Exception::ReferenceException);
	_INTERNAL ReferenceHolderBase(const std::string &ref, IType *root)
	throw(HGL::Exception::ReferenceException);
	_INTERNAL ReferenceHolderBase(const std::string &id, const std::string &dsc);
	ReferenceHolderBase(const ReferenceHolderBase &rh);

	~ReferenceHolderBase();

	/**
	 * @note This method returns @c true if a reference is set,
	 * but @b not checking if the reference itself is valid.
	 * @copydoc TypeBase::isValid
	 **/
	virtual bool isValid() const throw();
};

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase() : Base(), ReferenceHolderTBase() {}

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase(IType *lib) : Base(), ReferenceHolderTBase(lib) {}

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase(const std::string &ref)
throw(HGL::Exception::ReferenceException) : Base(), ReferenceHolderTBase(ref) {}

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase(const std::string &ref, IType *root)
throw(HGL::Exception::ReferenceException) : Base(root), ReferenceHolderTBase(ref) {}

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase(const std::string &id, const std::string &dsc) :
	Base(id, dsc), ReferenceHolderTBase() {}

template<class Base>
ReferenceHolderBase<Base>::ReferenceHolderBase(const ReferenceHolderBase &rh) : IType(),
	IReferenceHolder(), Base(rh), ReferenceHolderTBase(rh) {}

template<class Base>
ReferenceHolderBase<Base>::~ReferenceHolderBase() {}

template<class Base>
bool ReferenceHolderBase<Base>::operator==(const IType &t) const
throw(Exception::InvalidExpressionException) {
	return isPropertyEqual<IType>(&t);
}

template<class Base>
bool ReferenceHolderBase<Base>::isValid() const throw() {
	if(getReference()) {
		return true;
	}

	return Base::isValid();
}

template<>
bool IReferenceHolder::isPropertyEqual<HGL::IType>(const HGL::IType *) const throw();

extern template class ReferenceHolderBase<TypeBase>;

}

}

#endif /* HGL_TYPE_REFERENCEHOLDERBASE_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
