/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_REFERENCEHOLDERTBASE_H
#define HGL_TYPE_REFERENCEHOLDERTBASE_H

#include "ireferenceholder.h"

namespace HGL {

namespace Type {

class ReferenceHolderImpl;

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT ReferenceHolderTBase : public virtual IReferenceHolder {
	ReferenceHolderTBase &operator=(const ReferenceHolderTBase &);
public:
	using IType::operator=;

	_NOUNUSED virtual const std::string *getReference() const _HOT _PURE;

	virtual void setReference(const std::string &ref, bool fromStdLib = false)
	throw(Exception::ReferenceException);

	virtual void removeReference() throw(Exception::ReferenceException);

	_NOUNUSED virtual const IType *findReference(bool recache = false) const;

	static uint16_t getHashVal(const std::string &ref) _PURE;

protected:
	ReferenceHolderTBase();
	explicit ReferenceHolderTBase(IType *lib);
	_INTERNAL explicit ReferenceHolderTBase(const std::string &ref)
	throw(HGL::Exception::ReferenceException);
	_INTERNAL ReferenceHolderTBase(const ReferenceHolderTBase &);
	virtual ~ReferenceHolderTBase();

	virtual const IType *findReference(const std::string &ref, bool recache = false) const;

	virtual void setReference(const std::string &ref,
							  IType *scope) throw(Exception::ReferenceException);

	virtual void referenceHasChanged(const std::string &, const std::string &, bool) {}

private:
	/**
	 * @brief Calculates the hash value
	 *
	 * @param ref string to calculate hash value of
	 * @return uint16_t uint16_t the hash value
	 */
	virtual uint16_t getHash(const std::string &ref) const _PURE;
	virtual uint16_t getHash() const;

private:
	class ReferenceHolderImpl *const _pimpl;
};

}

}

#endif /* HGL_TYPE_REFERENCEHOLDERTBASE_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
