/*
    Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_REFERENCEPARAM_H
#define HGL_TYPE_REFERENCEPARAM_H

#include "referenceholderbase.h"
#include "parambase.h"
#include "serialid.h"

namespace HGL {

namespace ControlSource {
class ParameterFactory;
}

namespace Type {

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT ReferenceParam : public ReferenceHolderBase<TypeBase>, public ParamBase {
	DISALLOW_COPY_AND_ASSIGN(ReferenceParam)
	friend class HGL::ControlSource::ParameterFactory;
public:
	using ReferenceHolderBase<TypeBase>::findReference;
	using IParam::findReference;
	using ParamBase::operator==;
	using IType::operator*;
	using IType::operator=;

	ACCEPT(ReferenceParam);

	virtual const std::string toString() const;

	virtual bool operator==(const IType &t) const
	throw(Exception::InvalidExpressionException) {
		return ReferenceHolderBase<TypeBase>::operator==(t);
	}

	virtual operator float() const throw(Exception::InvalidExpressionException,
										 Exception::ReferenceException);

	/**
	 * @copydoc IReferenceHolder::findReference
	 **/
	_NOUNUSED virtual const IType *findReference(bool recache = false) const;

	virtual const SerialID &getSerialID() const throw() _CONST;

protected:
	ReferenceParam(const std::string &ref) throw(Exception::ReferenceException);
	ReferenceParam(const std::string &ref,
				   IType *root) throw(HGL::Exception::ReferenceException) _NONNULL(3);
	~ReferenceParam();

protected:
	std::string m_reference;
};

}

template<class Operator, class Fallback>
_INTERNAL IType *HGL::Type::IParam::operate(const IType &operand, Operator op,
		Fallback fallback) throw(Exception::InvalidExpressionException) {

	if(operand == SerialID::REFERENCEPARAM) {
		return op(this, reinterpret_cast<const HGL::Type::ReferenceParam *>(&operand));
	}

	return fallback(reinterpret_cast<TypeBase *>(this), operand);
}

template<> inline _HOT IType::Ptr::operator Type::ReferenceParam *() const throw() {
	return reinterpret_cast<Type::ReferenceParam *>(m_ptr);
}

extern template IType::Ptr::operator Type::ReferenceParam *() const throw();

}

#endif /* HGL_TYPE_REFERENCEPARAM_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
