/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGLSTRINGPARAM_H
#define HGLSTRINGPARAM_H

#include "parambase.h"

class EvaluationVisitorTest;
class ExpressionParamTest;

namespace HGL {

namespace ControlSource {
class ParameterFactory;
}

namespace Type {

/**
 * @ingroup types
 * @ingroup params
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT StringParam : public ParamBase {
	DISALLOW_COPY_AND_ASSIGN(StringParam)

	template<class T> friend IType::Ptr::operator T *() const throw();

	friend class HGL::ControlSource::ParameterFactory;
	friend class ::EvaluationVisitorTest;
	friend class ::ExpressionParamTest;
	friend class FloatParam;
	friend class NamedParam;
	friend class Condition;
	friend class Vector2D;

public:
	ACCEPT(StringParam);

	~StringParam();

	using IType::operator*;
	using IType::operator-;

	std::string getText() const;
	void setText(const std::string &txt);

	virtual const std::string toString() const;

	virtual StringParam &operator=(const IType &) throw(Exception::InvalidExpressionException);

	virtual operator StringParam &() const throw(Exception::InvalidExpressionException) {
		return const_cast<StringParam &>(*this);
	}

	IType *operator+(const IType &) throw(Exception::InvalidExpressionException);
	StringParam *operator-() throw(Exception::InvalidExpressionException);

protected:
	StringParam(const std::string &txt);

	/**
	 * @brief Gets the SerialID of HGL::IType::SSTRINGPARAM
	 *
	 * @return HGL::IType::SSTRINGPARAM
	 */
	const SerialID &getSerialID() const throw() _CONST;

private:
	std::string m_txt;
};

}

template<> inline _HOT IType::Ptr::operator Type::StringParam *() const throw() {
	return reinterpret_cast<Type::StringParam *>(m_ptr);
}

extern template IType::Ptr::operator Type::StringParam *() const throw();

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
