/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TEXT_H
#define HGL_TEXT_H

#include "proxy.h"
#include "colorbase.h"
#include "nonemptytype.h"

namespace HGL {

namespace Type {

/**
 * @ingroup types
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT _PACKED  Text : public Utility::Proxy<Text, NonEmptyType>, public ColorBase {
	DISALLOW_COPY_AND_ASSIGN(Text)
public:
	using IType::operator=;

	ACCEPT(Text);

	~Text();

	float getX() const;
	void setX(float x);

	float getY() const;
	void setY(float y);

	std::string getText() const;
	void setText(const std::string &txt);

	std::string getFont() const;
	void setFont(const std::string &font);

	uint32_t getFontSize() const;
	void setFontSize(uint32_t size);

	_NOUNUSED virtual const TYPELIST &getAttachedTypes() const throw();

	/**
	 * A @c Text is @b valid if the <em>first</em> and <em>second</em> attached type
	 * are @c StringParam and the <em>third</em> are a @c Vector2D.
	 * @copydoc NonEmptyType::isValid
	 **/
	virtual bool isValid() const throw();

	virtual BOUNDINGBOX getBoundingBox() const;

	virtual const SerialID &getSerialID() const throw() _CONST;

protected:
	Text();

	virtual const SerialID *const *getAllowedTypes() const throw();

private:
	StringParam *getTextSP() const;
	StringParam *getFontSP() const;
	Vector2D *getPointV2D() const;

private:
	uint32_t m_fontSize;
};

extern template class HGL::Type::Utility::Proxy<HGL::Type::Text, HGL::Type::NonEmptyType>;

}

template<> inline _HOT IType::Ptr::operator Type::Text *() const throw() {
	return reinterpret_cast<Type::Text *>(m_ptr);
}

extern template IType::Ptr::operator Type::Text *() const throw();

}

#endif // HGL_TEXT_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
