/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_ZSTREAMBUF_H
#define HGL_COMMON_ZSTREAMBUF_H

#include <streambuf>
#include <vector>

#include <zlib.h>

#include "zlibexception.h"

namespace HGL {

namespace Common {

class URI;

class _EXPORT Zstreambuf : public std::streambuf {
	DISALLOW_COPY_AND_ASSIGN(Zstreambuf)
public:
	Zstreambuf(const std::ostream &os, int compressionLevel = Z_DEFAULT_COMPRESSION,
			   bool flush = false) throw(Exception::ZLibException);
	explicit Zstreambuf(const URI *const uri) throw(Exception::ZLibException) _NONNULL(2);
	virtual ~Zstreambuf();

protected:
	virtual int sync();
	virtual int_type overflow(int_type);
	virtual int_type underflow();

private:
	int_type deflateBuffer(int flush);

private:
	z_stream m_zstream;
	static const size_t s_pback_size = 4;
	std::streambuf *m_sbuf;
	char_type *m_buf;
	bool m_inflating;
	uLong m_deflateBound;
	bool m_flush;
};

}

}

#endif /* HGL_COMMON_ZSTREAMBUF_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
